#from django.http import HttpResponse
from django.utils import simplejson as json
from django.http import HttpResponse
from dsympy.parser import SympyExprParser, SympyMethodParser
from dsympy.symbol_container import SymbolContainer
from dsympy.safe_eval import safe_eval
from dsympy.exceptions import JSONError
from dsympy import DSYMPY_DEBUG
import sympy, sys
from sympy.printing import pretty, mathml, latex

printer = pretty #alternatives: mathml, latex
symbol_container = SymbolContainer()

# error codes
UNKNOWN_ERROR = 0
NOT_VALID_JSON_RPC_ERROR = 1
NOT_VALID_METHOD_ERROR = 2
PYTHON_CONVERSION_ERROR = 3 # error converting to a python expression
METHOD_ARGS_ERROR = 4 # method expected a different set of arguments

TIMEOUT = 3 # seconds

class Result(object):
    result = None
    
    def clean(self):
        self.result = None

result = Result()

def json_rpc(request):
    """Process a JSON-RPC request. JSON data must be attached as a POST request.
    
    Conforming to the JSON-RPC specification, JSON data has three properties:
    
    * method - A String containing the name of the method to be invoked.
    * params - An Array of objects to pass as arguments to the method.
    * id - The request id. This can be of any type. It is used to match the response with the request that it is replying to.

    A typical remote method could be something like this: 
    { 
        "method" : "solve", 
        "params" : ["2 x + 1", "x"], 
        "id" : "null"
    }

    References
    ----------
        JSON-RPC specification: http://json-rpc.org/wiki/specification

    See also
    --------
        raw_json_rpc_process method
    """
    data = raw_json_rpc_process(request.raw_post_data)
    return HttpResponse(data, mimetype="application/json")

def xml_rpc(request):
    raise NotImplementedError

def raw_json_rpc_process(data):
    """Process a JSON-RPC Request
    
    Input is expected to be a string representing an object serialized to JSON.
    
    Examples
    --------
    >>> from django.utils import simplejson as json
    >>> request = json.dumps({ "method" : "solve", "params" : ["2 x + 1", "x"], "id" : "null" })
    >>> json_rpc_process(request)
    
    """
    if DSYMPY_DEBUG:
        print( "Received data: %s" % str(data) )
    data = json.loads(data)
    try:
        # check for valid json-rpc
        assert type(data) == dict
        keys = data.keys()
        for method in data.keys():
            assert method in [ 'method', 'id', 'params']
    except AssertionError:
        # Not a valid JSON-RPC request
        return json.dumps(
        { 
            'result': 'null', 
            'error': 
            { 
                'code': NOT_VALID_JSON_RPC_ERROR, 
                'message': 'not valid json-rpc' 
            },
            'id' : data['id']
        })

    method = SympyMethodParser(data['method']).parse()
    params = list()
    for d in data['params']: # first we extract symbols
        for sym in SympyExprParser(d).extract_symbols():
            if not sym in symbol_container:
                symbol_container[sym] = sympy.Symbol(sym)
    for d in data['params']:
        params.append( SympyExprParser(d).parse() )

    safe_eval("result.result = sympy.%s(*%s)" % (method, params), {'sympy':sympy, 'result':result})
    result.result = printer(result.result)
    
    if len(symbol_container) > 500:
        # don't let symbol_container grow too big
        symbol_container.reset()
    
    if isinstance(result, JSONError):
        # return error json
        raise NotImplementedError(result)

    if DSYMPY_DEBUG:
        print "json-rpc processing ended with result %s" % result

    return json.dumps({
            'result' : unicode(result.result),
            'error' : 'null',
            'id' : data['id']
            })
