
from dsympy.safe_eval import safe_eval
from dsympy.exceptions import SafeEvalException
import py.test
import time, __builtin__

def test_builtin():
    # attempt to access a unsafe builtin
    py.test.raises(SafeEvalException,
        """safe_eval("open('test_eval_expr.py', 'w')")""")

def test_getattr():
    # attempt to get arround direct attr access
    py.test.raises(SafeEvalException, \
        """safe_eval("getattr(int, '__abs__')")""")

def test_func_globals():
    # attempt to access global enviroment where fun was defined
    py.test.raises(SafeEvalException, \
        """safe_eval("def x(): pass; print x.func_globals")""")

def test_lowlevel():
    # lowlevel tricks to access 'object'
    py.test.raises(SafeEvalException, \
        """safe_eval("().__class__.mro()[1].__subclasses__()")""")

def test_timeout_ok():
    # attempt to exectute 'slow' code which finishes within timelimit
    def test(): time.sleep(2)
    env = {'test':test}
    safe_eval("test()", env, timeout_secs = 5)

def test_timeout_exceed():
    # attempt to exectute code which never teminates
    py.test.raises(SafeEvalException, \
        """safe_eval("while 1: pass")""")

def test_invalid_context():
    # can't pass an enviroment with modules or builtins
    env = {'f' : __builtin__.open, 'g' : time}
    py.test.raises(SafeEvalException, \
        """safe_eval( "print 1", env )""")

def test_eval_expr_1():
    py.test.raises(Exception, "safe_eval('factorint', '[345234523452353453*234423545244534535]'")
