import os
os.environ["DJANGO_SETTINGS_MODULE"] = "test_app.settings"

from django.test.client import Client
from django.utils import simplejson as json


def test_json_rpc_solve():
    data = { 
        "method" : "solve",
        "params" : ["2 x + 1", "x"],
        "id" : "null"
        }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] == u'[-1/2]'
    assert output['error'] == u'null'
    
    data = {
        "method" : "solve",
        "params" : ["x^2+ 1", "x"],
        "id" : "null"
    }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] == u'[-I, I]'
    assert output['error'] == u'null'
    
    data = {
        "method" : "solve",
        "params" : ["cos(x)", "x"],
        "id" : "null"
    }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] in [u'[pi/2]', u' pi \n[--]\n 2  ']
    assert output['error'] == u'null'

def test_json_rpc_diff():
    data = {
        "method" : "diff",
        "params" : ["x^2+ 1", "x"],
        "id" : "null"
    }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] == u'2*x'
    assert output['error'] == u'null'

# def test_json_rpc_diff_error_1():
#     data = {
#         "method" : "diff",
#         "params" : ["x^2+ 1"],
#         "id" : "null"
#     }
#     req = json.dumps(data)
#     c = Client()
#     response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
#     output = json.loads(response.content)
#     assert output['error'] == u'TODO'

def test_json_rpc_integrate():
    data = {
        "method" : "integrate",
        "params" : ["x^2+ 1", "x"],
        "id" : "null"
    }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] in [u'x + 1/3*x**3', u'     3\n    x \nx + --\n    3 ']
    assert output['error'] == u'null'

    data = {
        "method" : "integrate",
        "params" : ["log(x)", "x"],
        "id" : "null"
    }
    req = json.dumps(data)
    c = Client()
    response = c.post('/dsympy/jsonrpc/', data=req, content_type="text/json")
    output = json.loads(response.content)
    assert output['result'] == u'-x + x*log(x)'
    assert output['error'] == u'null'

def test_json_rpc_series():
    pass

def test_json_factorint():
    pass